@@ -204,7 +204,7 @@ def setLocusOfFocus(event, obj, notifyScript=True, force=False):
             debug.println(debug.LEVEL_INFO, msg, True)
             return
 
-    msg = "ORCA: Changing locusOfFocus from %s to %s" % (oldFocus, obj)
+    msg = "ORCA: Changing locusOfFocus from %s to %s. Notify: %s" % (oldFocus, obj, notifyScript)
     debug.println(debug.LEVEL_INFO, msg, True)
     orca_state.locusOfFocus = obj
 
@@ -379,6 +379,11 @@ def _restoreXmodmap(keyList=[]):
         stdin=subprocess.PIPE, stdout=None, stderr=None)
     p.communicate(_originalXmodmap)
 
+def setKeyHandling(new):
+    """Toggle use of the new vs. legacy key handling mode.
+    """
+    _eventManager.setKeyHandling(new)
+
 def loadUserSettings(script=None, inputEvent=None, skipReloadMessage=False):
     """Loads (and reloads) the user settings module, reinitializing
     things such as speech if necessary.
@@ -541,6 +546,20 @@ def helpForOrca(script=None, inputEvent=None, page=""):
                  Gtk.get_current_event_time())
     return True
 
+def addKeyGrab(binding):
+    """ Add a key grab for the given key binding. """
+    ret = []
+    for kd in binding.keyDefs():
+        ret.append(orca_state.device.add_key_grab(kd, None))
+    return ret
+
+def removeKeyGrab(id):
+    """ Remove the key grab for the given key binding. """
+    orca_state.device.remove_key_grab(id)
+
+def mapModifier(keycode):
+    return orca_state.device.map_modifier(keycode)
+
 def quitOrca(script=None, inputEvent=None):
     """Quit Orca. Check if the user wants to confirm this action.
     If so, show the confirmation GUI otherwise just shutdown.
@@ -682,6 +701,7 @@ def shutdown(script=None, inputEvent=None):
         signal.signal(signal.SIGALRM, settings.timeoutCallback)
         signal.alarm(settings.timeoutTime)
 
+    orca_state.activeScript.presentationInterrupt()
     orca_state.activeScript.presentMessage(messages.STOP_ORCA, resetStyles=False)
 
     _scriptManager.deactivate()
